#include "slang.h"
#include <io.h>

void Slang::slang_open()    // open(x, "work.txt", 1 + 3)
    {
    get_token();    // "("
    if(*token != '(')
	{ serror(1); return; }

    get_token();   // Name of handler variable
    if(token_type != VARIABLE)
	{ serror(2); return; }
    char* name = strdup(token);

    get_token();   // ','
    if(*token != ',')
	{ serror(0); return; }
    get_token();   // File name

    char* file_name;
    switch(token_type)
	{
	case QUOTE:
	    file_name = strdup(token);
	    break;
	case VARIABLE:
	    file_name = variables->find(token)->s;
	    if(file_name == NULL)
		{ delete name; serror(4); return; }
	    file_name = strdup(variables->find(token)->s);
	    break;
	default:
	    delete name;
	    serror(26);
	    return;
	}

    get_token(); // ","
    if(*token != ',')
	{ delete name; delete file_name; serror(0); return; }

    double value;   // Access string
    get_exp(&value);

    int handle = open(file_name, (int)value);
    variables->assign((double)handle, name);
    delete name;
    delete file_name;
    }
//////////////////////////
void Slang::slang_read()     // read(handle, buf <, len>)
    {
    get_token();    // "("
    if(*token != '(')
	{ serror(1);	return;	}
    get_token();   // Name of handler variable

    Variable* name;
    if(token_type != VARIABLE || !(name = variables->find(token)))
	{ serror(4); return; }

    get_token(); // ","
    if(*token != ',')
	{ serror(0); return; }

    get_token();   // Buf. name
    if(token_type != VARIABLE)
	{ serror(4); return; }
    char* buf_name = strdup(token);

    get_token(); // "," or ")"
    double value;              // Length
    switch(*token)
	{
	case ',':
	    if(get_exp(&value))
		{ delete buf_name; serror(0); return; }
	    break;
	case ')':
	    value = DEFAULT_STRING_LEN;
	    break;
	default:
	    serror(0);
	    return;
	}
    char* buf = new char[value + 1];

    int handle = name->d;
    for(int i = 0; i < value && !eof(handle); i++)
	{
	read(handle, buf + i, 1);
	if(buf[i] == '\n')
	    {
	    if(buf[i - 1] == '\r')
		i--;
	    break;
	    }
	}
    buf[i] = '\0';
    variables->assign(buf, buf_name);
    delete buf_name;
    delete buf;
    }
//////////////////////////
void Slang::slang_write()  // write(handle, buf <, len>)
    {
    get_token();    // "("
    if(*token != '(')
	{ serror(1);	return;	}
    get_token();
    Variable* name;
    if(token_type != VARIABLE || !(name = variables->find(token)))
	{ serror(4); return; }

    get_token(); // ","
    if(*token != ',')
	{ serror(0); return; }

    get_token();   // Buf. name
    Variable* buf_name;
    switch(token_type)
	{
	case VARIABLE:
	    if(!(buf_name = variables->find(token)))
		{ serror(4); return; }
	    break;
	case QUOTE:
	    buf_name = variables->assign(token, "retval");
//	    buf_name = variables->find("retval");
	    break;
	default:
	    serror(4);
	    return;
	}

    get_token(); // "," or ")"
    double value;              // Length
    switch(*token)
	{
	case ',':
	    if(get_exp(&value))
		{ serror(0); return; }
	    break;
	case ')':
	    value = strlen(buf_name->s);
	    break;
	default:
	    serror(0);
	    return;
	}

    write(name->d, buf_name->s, value);
    get_token();   // ";" or not
    if(*token == ';')
	write(name->d, "\r\n", 1);
    else
	putback();
    }
//////////////////////////
void Slang::slang_close()          // close
    {
    get_token();    // "("
    if(*token != '(')
	{ serror(1);	return;	}
    get_token();   // Name of handler variable
    Variable* v;
    if(!(v = variables->find(token)))
	{ serror(21); return; }
    close(v->d);
    }
//////////////////////////
int Slang::slang_eof()       // eof(handle)
    {
    get_token();    // "("
    if(*token != '(')
	{ serror(1);	return -1;	}
    get_token();   // Name of handler variable
    Variable* v;
    if(!(v = variables->find(token)))
	{ serror(21); return -1; }
    return eof(v->d);
    }
//////////////////////////
ulong Slang::slang_filelength()    // filelength(handle)
    {
    get_token();    // "("
    if(*token != '(')
	{ serror(1);	return -1;	}
    get_token();   // Name of handler variable
    Variable* v;
    if(!(v = variables->find(token)))
	{ serror(21); return -1; }
    return filelength(v->d);
    }
//////////////////////////
void Slang::slang_lseek()     // seek(handle, offset, fromwhere)
    {
    get_token();    // "("
    if(*token != '(')
	{ serror(1);	return;	}
    get_token();   // Name of handler variable
    if(token_type != VARIABLE)
	{ serror(2); return; }
    char* name = strdup(token);
    double offset, from;
    get_token();
    if(*token != ',')
	{ serror(0);	return;	}
    get_exp(&offset);
    if(*token != ',')
	{ serror(0);	return;	}
    get_exp(&from);
    Variable* v;
    if(!(v = variables->find(name)))
	{ serror(4); return; }
    lseek(v->d, offset, from);
    delete name;
    }
//////////////////////////
void Slang::slang_rename() // rename(oldname, newname)
    {
    get_token();    // "("
    if(*token != '(')
	{ serror(1);	return;	}

    double value;
    char* old_name = get_exp(&value);
    if(*token != ',')
	{ serror(0);	return;	}
    char* new_name = get_exp(&value);

    rename(old_name, new_name);
    }
//////////////////////////
void Slang::slang_setmode()  // setmode(handle, 1+3+5)
    {
    get_token();    // "("
    if(*token != '(')
	{ serror(1);	return;	}
    get_token();   // Name of handler variable
    if(token_type != VARIABLE)
	{ serror(2); return; }
    char* name = strdup(token);
    get_token();
    if(*token != ',')
	{ delete name; serror(0);	return;	}

    double mode;
    get_exp(&mode);
    Variable* v;
    if(!(v = variables->find(name)))
	{ serror(4); return; }
    setmode(v->d, mode);
    delete name;
    }
//////////////////////////
ulong Slang::slang_tell()  // getpos(handle)
    {
    get_token();    // "("
    if(*token != '(')
	{ serror(1);	return -1;	}

    get_token();
    if(token_type != VARIABLE)
	{ serror(2); return -1; }
    Variable* v;
    if(!(v = variables->find(token)))
	{ serror(4); return -1; }
    return tell(v->d);
    }
//////////////////////////
void Slang::slang_unlink()   // remove(name)
    {
    get_token();    // "("
    if(*token != '(')
	{ serror(1);	return;	}

    get_token();
    if(token_type != VARIABLE || token_type != QUOTE)
	{ serror(2); return; }
    double value;
    remove(get_exp(&value));
    }
//////////////////////////
void Slang::slang_get_token() // gettoken(string, token), tokens_string is
    {                         // , ;TAB('\9')
    get_token();    // "("
    if(*token != '(')
	{ serror(1);	return;	}
    Variable* s;
    get_token();
    switch(token_type)
	{
	case VARIABLE:
	    if(!(s = variables->find(token)))
		{ serror(4); return; }
	    break;
	default:
	    serror(2);
	    return;
	}

    get_token();     // ',' or ')'
    switch(*token)
	{
	case ',':
	    get_token();
	    if(token_type != VARIABLE)
		{ serror(2); return; }
	    break;
	case ')': break;
	default: serror(0); return;
	}

    char* tmp = s->s;
    int l = strlen(s->s);
    int b = 0;
    variables->assign((double)1, "retval");
    for(int i = 0; i < l; i++)
	{
	while((tmp[i] == ' ' || tmp[i] == ',' || tmp[i] == ';' || tmp[i] == '\t')
	    && i < l)
	    {
	    tmp[i++] = '\0';
	    b = 1;
	    }
	if(b)
	    break;
	}
    variables->assign(tmp, token);
    char* tp = strdup(s->s + i);
    delete s->s;
    s->s = tp;
    if(i == l)
	variables->assign((double)0, "retval");
    }
//////////////////////////
double Slang::slang_get_real(Variable* s)
    {
    char* tmp = s->s;
    int l = strlen(s->s);
    int b = 0;
    variables->assign((double)1, "retval");
    for(int i = 0; i < l; i++)
	{
	while((tmp[i] == ' ' || tmp[i] == ',' || tmp[i] == ';' || tmp[i] == '\t')
	    && i < l)
	    {
	    tmp[i++] = '\0';
	    b = 1;
	    }
	if(b)
	    break;
	}
    char* endptr;
    double d = strtod(tmp, &endptr);
    if(d == HUGE_VAL)
	d = 0;
    char* tp = strdup(s->s + i);
    delete s->s;
    s->s = tp;
    if(i == l)
	variables->assign((double)0, "retval");
    return d;
    }
//////////////////////////
void Slang::slang_put_token()    // gettoken(string, token)
    {
    get_token();    // "("
    if(*token != '(')
	{ serror(1);	return;	}
    Variable* tok_v;
    Variable* str_v;
    get_token();                     // String
    if(token_type != VARIABLE)
	{ serror(2); return; }
    if(!(str_v = variables->find(token)) || (str_v && str_v->type != STR))
	variables->assign("", token);

    get_token(); // ','
    if(*token != ',')
	{ serror(1);	return;	}
    get_token();                     // Token
    int sl = strlen(str_v->s);
    int tl;
    switch(token_type)
	{
	case QUOTE:
	    tl = strlen(token);
	    str_v->s = (char*)realloc(str_v->s, sl + tl + 1);
	    strcpy(str_v->s + sl, token);
	    break;
	case VARIABLE:
	    tok_v = variables->find(token);
	    if(tok_v == NULL || (tok_v && tok_v->type != STR))
		{ serror(4); return; }
	    tl = strlen(tok_v->s);
	    str_v->s = (char*)realloc(str_v->s, sl + tl + 1);
	    strcpy(str_v->s + sl, tok_v->s);
	    break;
	default:
	    serror(26);
	    return;
	}
    }
//////////////////////////
void Slang::next_line()
    {
    get_token();    // "("
    if(*token != '(')
	{ serror(1);	return;	}
    Variable* v;
    get_token();                     // String
    if(token_type != VARIABLE || !(v = variables->find(token))
	|| (v->type != REAL))
	{ serror(2); return; }
    char buf[2];
    while(1)
	{
	read(v->d, buf, 1);
	if(buf[0] == '\n' || eof(v->d))
	    break;
	}
    }
/////////////////////////////